//! Game Parser and Data types
use std::marker::PhantomData;

use character_sheet::Character;
use serde::Serialize;

pub mod character_sheet;
pub mod chat_message;
pub mod entry;

pub trait GameImpl<C: Character<A> + Serialize, A: entry::GameEntry + Serialize> {
    /// Creates a new game
    fn new() -> Self;
    /// Creates a new character, returning the character id
    fn create_character(&mut self) -> usize;
}

pub struct Game<C: Character<A> + Serialize, A: entry::GameEntry + Serialize> {
    _c: PhantomData<C>,
    _a: PhantomData<A>,
    characters: Vec<C>,
}
impl<C: Character<A> + Serialize, A: entry::GameEntry + Serialize> GameImpl<C, A> for Game<C, A> {
    fn new() -> Self {
        Self {
            _c: PhantomData,
            _a: PhantomData,
            characters: Vec::new(),
        }
    }

    fn create_character(&mut self) -> usize {
        self.characters.push(C::default());
        self.characters.len() - 1
    }
}